;; -*- mode: scheme; coding: utf-8 -*-

;;;; Copyright (C) 2011, 2012
;;;; Free Software Foundation, Inc.
;;;;
;;;; This library is free software; you can redistribute it and/or
;;;; modify it under the terms of the GNU General Public License as
;;;; published by the Free Software Foundation; either version 3 of
;;;; the License, or (at your option) any later version.
;;;; 
;;;; This library is distributed in the hope that it will be useful,
;;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;;;; General Public License for more details.
;;;; 
;;;; You should have received a copy of the GNU General Public License
;;;; along with this library; if not, write to the Free Software
;;;; Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
;;;; 02110-1301 USA
;;;;

;;; Commentary:

;;; Code:

(define-module (nbs fp)
  ;; guile
  :use-module (ice-9 format)

  :export (*fp/default-precision*
	   fp/zero?
	   fp/=?
	   fp/<?
	   ;; float<=?
	   ;; float>?
	   ;; float>=?
	   fp/round-1
	   fp/round))

;(defun short-float/= (f1 f2 &optional (precision *precision*))
;  (declare (inline short-float/=)
;	   (short-float f1 f2 precision)
;	   (optimize (safety 0) (space 0) (speed 3) (debug 0)))
;  (> (abs (- f1 f2)) precision))

;(defun short-float< (f1 &optional (precision *precision*))
;  (declare (inline short-float/=)
;	   (short-float f1 f2 precision)
;	   (optimize (safety 0) (space 0) (speed 3) (debug 0)))
;  (< (abs f1) precision))

(define *fp/default-precision* 1.0E-4)

(define (fp/zero? a-float . prec)
  (if (null? prec)
      (<= (abs a-float) *fp/default-precision*)
      (<= (abs a-float) (car prec))))

(define (fp/=? f1 f2 . prec)
  (if (null? prec)
      (<= (abs (- f1 f2)) *fp/default-precision*)
      (<= (abs (- f1 f2)) (car prec))))

(define (fp/<? f1 f2 . prec)
  (let ((diff (- f1 f2)))
    (if (null? prec)
	(and (negative? diff)
	     (> (abs diff) *fp/default-precision*))
	(and (negative? diff)
	     (> (abs diff) (car prec))))))

(define (fp/round-1 float dec-nb)
  (format #f "~?" (format #f "~~,~AF" dec-nb)
	  (list float)))

(define (fp/round float . dec)
  (if (null? dec)
      (string->number (fp/round-1 float 2))
      (string->number (fp/round-1 float (car dec)))))


#!

(use-modules (nbs fp))
(reload-module (resolve-module '(nbs fp)))


(fp/zero? 0.001)
(fp/zero? 0.0001)

(fp/=? 0.001 0.0001)
(fp/=? 0.0001 0.0001)
(fp/=? 0.0001 -0.0001)

(define f1 0.0001)
(define f2 0.001)
(fp/<? f1 f2)

(fp/round 13.3456723 3)

!#

